/*
 * PM - 2017
 * lab0.c
 */

#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "usart.h"
#include "onewire.h"
#include "ds18b20.h"
#include "romsearch.h"

void ADC_Init() {
	DDRC = 0x0;		/*  Make ADC port as input  */
	ADCSRA = 0x87;		/*  Enable ADC, fr/128  */	
}

int ADC_Read() {
	ADMUX = 0x40;		/* Vref: Avcc, ADC channel: 0  */
	ADCSRA |= (1<<ADSC);	/* start conversion  */
	while ((ADCSRA &(1<<ADIF))==0);	/* monitor end of conversion interrupt flag */
	ADCSRA |=(1<<ADIF);	/* set the ADIF bit of ADCSRA register */
	return(ADCW);		/* return the ADCW */
}

void get_temp() {
	int temp;
	float celsius;
	char buffer[6];

	ds18b20convert( &PORTB, &DDRB, &PINB, ( 1 << 0 ), NULL );
	_delay_ms( 1000 );
	ds18b20read( &PORTB, &DDRB, &PINB, ( 1 << 0 ), NULL, &temp );

	celsius = temp / 16.0;
	USART0_print("Temperatura aer: ");
	dtostrf(celsius, 2, 2,buffer);
	strcat(buffer, " C\n");
	USART0_print(buffer);
	memset(buffer, 0, 6);
}

void get_humidity() {
	char array[10];
	int adc_value;
	float moisture;

	adc_value = ADC_Read();
	moisture = 100 - (adc_value * 100.00) / 1023.00;
	USART0_print("Umiditate sol: ");
	dtostrf(moisture,3,2,array);
	strcat(array,"%   \n");
	USART0_print(array);
	memset(array, 0, 10);
}

void get_light() {
	char array[15];
	int adc_value;
	float light, voltage, lux_value;

	adc_value = ADC_Read();
	voltage = (5.00 * adc_value) / 1023.00;

	light = (voltage / 10000.0) * 1000000; 
	lux_value = 5.0 * light;

	USART0_print("Intensitate luminoasa: ");
	dtostrf(lux_value, 8, 4, array);
	USART0_print(array);
	USART0_print(" LUX\n");
	memset(array, 0, 15);
}

int main(void) {
	ADC_Init();
	USART0_init();

	while(1) {
		char c = USART0_receive();

		switch (c) {
			case 't':
				USART0_print("Afiseaza temperatura\n");
				get_temp();
				break;
			case 'u':
				USART0_print("Afiseaza umiditatea solului\n");
				get_humidity();
				break;
			case 'l':
				USART0_print("Afiseaza intensitatea luminii\n");
				get_light();
				break;
		}
	}

	return 0;
}