/*
 * Proiect PM - 2019
 * Gazdaru Alina Alexandra
 * 331CB
 */

#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "usart.h"
#include "onewire.h"
#include "ds18b20.h"
#include "romsearch.h"

#define light_minimum 1000
#define soil_humidity_minimum 50
#define temperature_minimum 10

// convertor analog digital
// colecteaza date pe canalul 0
void ADC_Init() {
	DDRC = 0x0;	
	ADCSRA = 0x87;	
}


// preluarea datelor analog
int ADC_Read() {
	ADMUX = 0x40;
	ADCSRA |= (1<<ADSC);
	while ((ADCSRA &(1<<ADIF))==0);
	ADCSRA |=(1<<ADIF);	
	return(ADCW);
}

// functie ce returneaza temperatura aerului
void get_temp() {
	int temp;
	float celsius;
	char buffer[6];

    // colectarea si convertirea valorilor	
    // initializare sensor si convertire date
    ds18b20convert( &PORTB, &DDRB, &PINB, ( 1 << 0 ), NULL );
	_delay_ms( 1000 ); // se asteapta convertirea
    // citirea datelor intr-o variabila temp
	ds18b20read( &PORTB, &DDRB, &PINB, ( 1 << 0 ), NULL, &temp );

    // transformarea in grade celsius
	celsius = temp / 16.0;
	USART0_print("Temperatura aer: ");
	dtostrf(celsius, 2, 2,buffer);
	strcat(buffer, " C\n");
	USART0_print(buffer);
	memset(buffer, 0, 6);

	if (celsius < temperature_minimum) {
		USART0_print("Temperatura prea scazuta. Muta intr-o zona mai calduroasa.\n");
	}
}

// temperatura ce returneaza nivelul de umiditate a solului in procentaje
void get_humidity() {
	char array[10];
	int adc_value;
	float moisture;

	DDRD |= (1 << PD7);
	PORTD |= (1 << PD7);

	adc_value = ADC_Read();
	moisture = 100 - (adc_value * 100.00) / 1023.00;
	USART0_print("Umiditate sol: ");
	dtostrf(moisture,3,2,array);
	strcat(array,"%   \n");
	USART0_print(array);
	memset(array, 0, 10);

    // in cazul in care este prea mic, se porneste pompa	
    if (moisture < soil_humidity_minimum) {
		USART0_print("Umiditatea solului e prea scazuta. Porneste pompa.\n");
		PORTD &= ~(1 << PD7);
	} else {
		PORTD |= (1 << PD7);
	}
}


// functie ce calculeaza intensitatea luminoasa
void get_light() {
	char array[15];
	int adc_value;
	float light, voltage, lux_value;

	adc_value = ADC_Read();
	voltage = (5.00 * adc_value) / 1023.00;

	light = (voltage / 10000.0) * 1000000; 
	lux_value = 5.0 * light;

	USART0_print("Intensitate luminoasa: ");
	dtostrf(lux_value, 8, 4, array);
	USART0_print(array);
	USART0_print(" LUX\n");
	memset(array, 0, 15);

	if (lux_value < light_minimum) {
		USART0_print("Lumina prea putina. Muta intr-o zona mai luminoasa.\n");
	}
}

int main(void) {
	ADC_Init();
	USART0_init();
	sei();

	while(1) {
    // comenzi primite de la utilizator
    // comunicare cu modulul de bluetooth HC-05
		char c = USART0_receive();

		switch (c) {
			case 't':
				USART0_print("Afiseaza temperatura\n");
				get_temp();
				break;
			case 'u':
				USART0_print("Afiseaza umiditatea solului\n");
				get_humidity();
				break;
			case 'l':
				USART0_print("Afiseaza intensitatea luminii\n");
				get_light();
				break;
		}
	}

	return 0;
}
